/*
 * libmedia-thumbnail
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Hyunjun Ko <zzoon.ko@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include "media-thumbnail.h"
#include "media-thumb-debug.h"
#include "media-thumb-util.h"
#include "media-thumb-internal.h"
#include "media-thumb-ipc.h"

#include <glib.h>

int thumbnail_request_from_db_async(unsigned int request_id, const char *origin_path, ThumbFunc func, void *user_data, uid_t uid)
{
	int err = MS_MEDIA_ERR_NONE;
	ms_user_storage_type_e store_type = MS_USER_STORAGE_INTERNAL;

	thumb_retvm_if(!origin_path, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid parameter");
	thumb_retvm_if(!g_file_test(origin_path, G_FILE_TEST_IS_REGULAR), MS_MEDIA_ERR_INVALID_PARAMETER, "No file [%s]", origin_path);

	err = ms_user_get_storage_type(uid, origin_path, &store_type);
	if ((err != MS_MEDIA_ERR_NONE) || ((store_type != MS_USER_STORAGE_INTERNAL) && (store_type != MS_USER_STORAGE_EXTERNAL))) {
		thumb_err_slog("origin path(%s) is invalid. err : [%d] store_type [%d]", origin_path, err, store_type);
		return MS_MEDIA_ERR_INVALID_PARAMETER;
	}

	thumb_dbg_slog("Path : %s", origin_path);

	thumbUserData *userData = (thumbUserData*)malloc(sizeof(thumbUserData));
	thumb_retvm_if(!userData, MS_MEDIA_ERR_OUT_OF_MEMORY, "Allocation failed");

	userData->func = (ThumbFunc)func;
	userData->user_data = user_data;

	/* Request for thumb file to the daemon "Thumbnail generator" */
	err = _media_thumb_request_async(THUMB_REQUEST_DB_INSERT, request_id, origin_path, userData, uid);
	if (err != MS_MEDIA_ERR_NONE) {
		thumb_err("_media_thumb_request failed : %d", err);
		SAFE_FREE(userData);
		return err;
	}

	return MS_MEDIA_ERR_NONE;
}

int thumbnail_request_extract_raw_data_async(int request_id, const char *origin_path, int width, int height, ThumbRawFunc func, void *user_data, uid_t uid)
{
	int err = MS_MEDIA_ERR_NONE;
	int exist = 0;

	thumb_retvm_if(!origin_path || request_id == 0, MS_MEDIA_ERR_INVALID_PARAMETER, "original path is NULL. Or there is an error in request_id.");

	/* check the file exits actually */
	exist = open(origin_path, O_RDONLY);
	if (exist < 0) {
		if (errno == EACCES || errno == EPERM) {
			thumb_err("Fail to open original_path[%s]: Permission Denied", origin_path);
			return  MS_MEDIA_ERR_PERMISSION_DENIED;
		} else {
			thumb_err("Fail to open original_path[%s]: Invalid Path", origin_path);
			return MS_MEDIA_ERR_INVALID_PARAMETER;
		}
	}
	close(exist);

	thumb_dbg_slog("Path : %s", origin_path);

	thumbRawUserData *userData = (thumbRawUserData*)malloc(sizeof(thumbRawUserData));
	thumb_retvm_if(!userData, MS_MEDIA_ERR_OUT_OF_MEMORY, "Allocation failed");

	userData->func = func;
	userData->user_data = user_data;

	err = _media_thumb_request_raw_data_async(THUMB_REQUEST_RAW_DATA, request_id, origin_path, width, height, userData, uid);
	if (err != MS_MEDIA_ERR_NONE) {
		thumb_err("_media_raw_thumb_request failed : %d", err);
		SAFE_FREE(userData);
		return err;
	}

	return MS_MEDIA_ERR_NONE;
}

int thumbnail_request_cancel_media(unsigned int request_id)
{
	int err = MS_MEDIA_ERR_NONE;

	err = _media_thumb_request_async(THUMB_REQUEST_CANCEL_MEDIA, request_id, NULL, NULL, 0);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "_media_thumb_request_async failed : %d", err);

	return err;
}

int thumbnail_request_cancel_raw_data(int request_id)
{
	int err = MS_MEDIA_ERR_NONE;

	thumb_retvm_if(request_id == 0, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid parameter");

	err = _media_thumb_request_raw_data_async(THUMB_REQUEST_CANCEL_RAW_DATA, request_id, NULL, 0, 0, NULL, 0);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "_media_thumb_request_raw_data_async failed : %d", err);

	return err;
}
